  <ul class="list-group">
    <a href="/settings/instances"
       class="list-group-item">Instances</a>
    </a>
    <a href="/settings/groups/instance"
       class="list-group-item">Instance groups</a>
    </a>
    <a href="/settings/users"
       class="list-group-item">Users</a>
    </a>
    <a href="/settings/groups/role"
       class="list-group-item">User groups</a>
    </a>
    <a href="/settings/notifications"
       class="list-group-item">Alert notifications</a>
    </a>
  </ul>
