{% extends ../base.html %}

{% block title %}Temboard - Error{% end %}

{% block content %}
<div class="row">
  <div class="col-4 offset-4">
    <div class="alert alert-danger alert-restricted" role="alert">
      <h4><i class="fa fa-ban fa-fw"></i> Error <small></small></h4>
      <p>{{ error }}</p>
      <center><a class="btn btn-sm btn-danger" href="/">Back to home page</a></center>
    </div>
  </div>
  <div class="col-4"></div>
</div>
{% end %}
