$icon-font-path: "../fonts/" !default;
$primary: #3c424c;
$success: #1d9d74;
$undef: #ccc;
$cat1: #78B8FA;
$cat2: #FA9E78;
$navbar-inverse-bg: $primary;
$link-color: lighten(saturate($primary, 15%), 15%);

$input-border-focus: $link-color;
$grid-gutter-width: 20px;
$spacer: $grid-gutter-width;
$small-font-size: 85%;

// Additional xxl grid tiers
$grid-breakpoints: (
  xs: 0,
  sm: 576px,
  md: 768px,
  lg: 992px,
  xl: 1200px,
  xxl: 1500px
) !default;

$red: #dc3545 !default;
$danger: $red !default;

$theme-colors: (
  "ok": $success,
  "critical": $danger,
  "undef": $undef,
  "cat1": $cat1,
  "cat2": $cat2,
) !default;
$popover-font-size: 1rem !default;
