$daterangepicker-active-bg-color: $primary;
$daterangepicker-ranges-hover-bg-color:  $primary;
$daterangepicker-control-active-border-color:  $primary;
$daterangepicker-ranges-color: $primary;
$daterangepicker-cell-hover-bg-color: lighten($primary, 50%);
$daterangepicker-in-range-bg-color: lighten($primary, 60%);
$daterangepicker-z-index: $zindex-dropdown !default;

.daterangepicker {
  z-index:1151 !important;
}
