{% extends base.html %}

{% block title %}temBoard - Login{% end %}

{% block content %}
<div class="row justify-content-center">
  <div class="col-12 col-md-6 col-lg-4">
    <div class="card" id="panelLogin">
      <div class="card-header bg-primary text-center">
        <img src="/images/temboard-150x32-w.png" />
      </div>
      <div class="card-body">
        {% try %}{% if error %}<div class="alert alert-danger" role="alert">{{ error }}</div>{% end %}{% except %}{% end %}
        <form role="form" action="/login" method="post">
          <div class="form-group">
            <label for="inputUsername" class="control-label">Username</label>
            <input type="text" name="username" class="form-control" id="inputUsername" placeholder="Username" autofocus>
          </div>
          <div class="form-group">
            <label for="inputPassword" class="control-label">Password</label>
            <input type="password" name="password" class="form-control" id="inputPassword" placeholder="Password">
          </div>
          <div class="text-center">
            <button type="submit" class="btn btn-primary">Login</button>
          </div>
        </form>
      </div>
    </div>
  </div>
</div>
{% end %}
